#include <xc.h>
#include "hw.h"


// CONFIG1
#pragma config FOSC = INTOSC    // Oscillator Selection Bits (INTOSC oscillator: I/O function on CLKIN pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable (PWRT enabled)
#pragma config MCLRE = ON       // MCLR Pin Function Select (MCLR/VPP pin function is MCLR)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config BOREN = ON       // Brown-out Reset Enable (Brown-out Reset enabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
#pragma config IESO = ON        // Internal/External Switchover Mode (Internal/External Switchover Mode is enabled)
#pragma config FCMEN = ON       // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is enabled)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LPBOR = OFF      // Low-Power Brown Out Reset (Low-Power BOR is disabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)

unsigned int tst;
void hw_init (void)
{
TRISB6 = 0;
TRISC7 = 0;
SSP1CON1 = 0x21;
SSP1STAT = 0x40;
ANSELA = 0x00;
ANSELB = 0x00;
OSCCON = 0x78;
LED_T = 0;
TRST_T = 0;
MCS_T = 0;
MCS = 1;
TXSTA = 0x24;
RCSTA = 0x90;
BRG16 = 1;
SPBRGL = (_XTAL_FREQ/BAUD_RATE/4) - 1;
FVRCON = 0x81;
ADCON0 = 0x7D;
ADCON1 = 0xF0;
}


unsigned int get_vdd (void)
{
unsigned long res,res2;
GO_nDONE = 1;
while (GO_nDONE==1);
res2 = (((unsigned long)(ADRESH))<<8) | ADRESL;
res = 1023;
res = res * 1024;
res = res / res2;
return res;
}


//**********************************************************

unsigned char hwspi_byte(unsigned char data)
{
led(1);
SSP1IF=0;
SSPBUF = data;
while (SSP1IF==0);
led(0);
return SSPBUF;
}

void mem_start_write (unsigned int addr)
{
MCS = 0;
hwspi_byte(0x02);
hwspi_byte(0x00);
hwspi_byte(addr>>8);
hwspi_byte(addr>>0);
}

void mem_write_byte (unsigned char data)
{
hwspi_byte(data);
}

void mem_end_write (void)
{
MCS = 1;
}

void mem_read_range (unsigned int addr, unsigned char * buf, unsigned char len)
{
unsigned char i;
MCS = 0;
hwspi_byte(0x03);
hwspi_byte(0x00);
hwspi_byte(addr>>8);
hwspi_byte(addr>>0);
for (i=0;i<len;i++)
	buf[i] = hwspi_byte(0x00);
MCS = 1;
}

//**********************************************************

unsigned char rx_ready (void)
{
if (RCIF)
	return 1;
else
	return 0;
}

unsigned char rx_get (void)
{
return RCREG;
}

void tx_str (char * data)
{
while (*data!=0) tx_data(*data++);
}

void tx_data (unsigned char dat)
{
while (TRMT==0);
TXREG = dat;
}

//**********************************************************

void enter_progmode_pins (unsigned char mode, unsigned char rst_log)
{
TMISO_T = 1;
TMOSI_T = 0;
TCS_T = 0;
TSCK_T = 0;
if (rst_log) TRST = 1;
	else TRST = 0;
if (mode==0)
	{
	TSCK = 0;
	dly_ms(5);
	if (rst_log) TRST = 0;
		else TRST = 1;
	dly_ms(5);
	}
else if ((mode==1)|(mode==2))
	{
	set_cs(1);
	TSCK = 1;
	dly_ms(5);
	if (rst_log) TRST = 0;
		else TRST = 1;
	dly_ms(5);
	TSCK = 0;
	dly_ms(5);
	}
}

void exit_progmode_pins (unsigned char rst_log)
{
set_cs(1);
TMISO_T = 1;
TMOSI_T = 1;
TSCK_T = 1;
dly_ms(5);
if (rst_log) TRST = 1;
	else TRST = 0;
TCS_T = 1;
}

void dly_ms(unsigned int ms)
{
unsigned int i;
for (i=0;i<ms;i++) __delay_ms(1);
}

unsigned char spi_byte (unsigned char dat)
{
unsigned char i,data;
data =0;
led(1);
for (i=0;i<8;i++)
	{
	if (dat&0x80)
		TMOSI = 1;
	else
		TMOSI = 0;
	__delay_us(2);
	TSCK = 1;
	__delay_us(2);
	data = data << 1;
	dat = dat << 1;
	if (TMISO_I!=0)
		data = data|0x01;
	TSCK = 0;

	}
led(0);
return data;
}

void led (unsigned char data)
{
if (data==0)
	LED = 0;
else
	LED = 1;
}

void set_cs (unsigned char data)
{
if (data==0)
	TCS = 0;
else
	TCS = 1;
}

